--[[
**************************
Script version: v0.90 beta
Requires CHDK 1.3!
Licence: GPL
(c) Werner_O 2014/11/15
Thx msl
**************************
@title Test focus
@param    d Start Distance (mm)
 @default d 50000
 @range   d 0 65000
@param    s Step-size at Start (mm)
 @default s 3
 @values  s 1 10 100 1000 10000 100000 1000000
@param    a Max. Digits AutoInfMode
 @default a 5
 @range   a 3 5
@param    t StepTime AutoInfMode (s)
 @default t 3
 @range   t 1 6
@param    v Value for infinite
 @default v 0
 @values  v -1 65535
@param    m Test infinite only
 @default m 0
 @range   m 0 1
]]

--Initialize used variables:
start_distance = d
digits = a
steps = math.pow(10, s)   --Generate first available step-size
mf_mode = 0
afl_mode = 0
show_title = true
show_meter = true
inf_test = m
if v == 0 then            --Generate value for infinite
    inf = -1
else
    inf = 65535
end
step_time = t * 1000
current_zoom = get_zoom()           --Store current zoom position
max_zoom = (get_zoom_steps() - 1)   --Find max. zoom position
current_inf = -2
current_maxfoc = -2
error_beep = true
ready = true

--Define console layout:
set_console_layout(0, 0, 32, 12)

--Subfunction set_fixfoc() to set a focus and show the results:
function set_fixfoc()
    local count = 0
    local timeout = false
    set_focus(distance)
    press("shoot_half")
    repeat
        sleep(100)
        count = count + 1
        if count > 50 then timeout = true end
    until get_shooting() or timeout
    release("shoot_half")
    focus = get_focus()
    if show_title == true then
        message = "Wanted focus:"
        x = distance
        print_focus()
    end
    if show_meter == true then
        message = "Focus set to:"
        x = focus
        print_focus()
    end
end

--Subfunction print_focus() to show wanted/realized distances:
function print_focus()
    x_string = tostring(x)
    if (x == -2) then           --True at missing valid value
        print(string.format("%s NOT TESTED!", message))
    elseif (x == -1) then       --True at distance infinite
        print(string.format("%s infinite", message))
    elseif ((inf == 65535) and (x == 65535)) then
        print(string.format("%s infinite", message))
    elseif (x < 1000) then      --True at distances < 1 m
        print(string.format("%s %03d mm", message, focus))
    elseif (x < 10000) then     --True at distances >= 1 m to < 10 m
        local meter = tonumber(string.sub(x_string, 1, 1))
        local mm = string.sub(x_string, 2, 4)
        print(string.format("%s %04d,%s m", message, meter, mm))
    elseif (x < 100000) then    --True at distances >= 10 m to < 100 m
        local meter = tonumber(string.sub(x_string, 1, 2))
        local mm = string.sub(x_string, 3, 5)
        print(string.format("%s %04d,%s m", message, meter, mm))
    elseif (x < 1000000) then   --True at distances >= 100 m to < 1000 m
        local meter = tonumber(string.sub(x_string, 1, 3))
        local mm = string.sub(x_string, 4, 6)
        print(string.format("%s %04d,%s m", message, meter, mm))
    else                        --True at distances >= 1000 m
        local meter = tonumber(string.sub(x_string, 1, 4))
        local mm = string.sub(x_string, 5, 7)
        print(string.format("%s %04d,%s m", message, meter, mm))
    end
end

--Subfunction print_steps to show current step size:
function print_steps()
    print(string.format("Steps: %07d mm", steps))
end

--Subfunction show_state to show current script state:
function show_state()
    cls()
    print(string.format("Current inf-value: %d", inf))
    print_steps()
    message = "Last real inf:"
    x = current_inf
    print_focus()
    message = "Last real MaxFoc:"
    x = current_maxfoc
    print_focus()
    message = "Wanted focus:"
    x = distance
    print_focus()
    message = "Focus set to:"
    x = focus
    print_focus()
    sleep(3000)
    console_redraw()
end

--Subfunction increase_steps():
function increase_steps()
    if steps < 1000000 then
       local s_factor = tonumber(string.sub(tostring(steps), 1, 1))
       if s_factor == 1 then
           steps = steps * 5
       else
           steps = steps * 2
       end
    end
    print(string.format("Steps: %07d mm", steps))
end

--Subfunction decrease_steps():
function decrease_steps()
    if steps > 1 then
       local s_factor = tonumber(string.sub(tostring(steps), 1, 1))
       if s_factor == 1 then
           steps = steps / 2
       else
           steps = steps / 5
       end
    end
    print(string.format("Steps: %07d mm", steps))
end 

--Subfunction test_inf() to get value for infinite:
function test_inf()
    cls()
    print("Start test for infinite-value:")
    print("MACRO-MODE MUST BE DISABLED")
    print("FOR THIS TEST!")
    print("Press DISPLAY to continue test:")
    print("Press MENU to abort test:")
    repeat           
        wait_click(2000)
        if is_pressed("display") then
            cls()
            print("Searching value for inf:")
            set_zoom(max_zoom)                        --Set max. zoom position
            sleep(5000)
            distance = 9000000
            show_title = true
            show_meter = true
            set_fixfoc()
            inf_string = tostring(focus)
            set_zoom(current_zoom)                    --Restore former zoom position
            inf_test = -1
            cls()
            errormessage1 = "Value infinite must be set to:"
            errormessage2 = inf_string
        elseif is_pressed("menu") then
            inf_test = -1
            cls()
            errormessage1 = "The test for infinite has"
            errormessage2 = "been aborted!"
        end
        console_redraw()
    until inf_test == -1
    error_beep = false
end

--Subfunction auto_inf_search() to auto-search infinite distance and max. focal distance below infinite:
function auto_inf_search()
    cls()
    show_title = false
    show_meter = false
    print("Auto-mode activated,")   
    print("searching for max. focus:")
    i = 0
    start_distance = 0
    if inf == -1 then
        steps = 10000000     --Will be changed the first time 7 lines below
    else
        steps = 100000       --Will be changed the first time 5 lines below
    end
    repeat
        distance = 10000
        set_fixfoc()         --Reset focus to a near and always valid start value (10m)
        steps = steps / 10   --Change to next lower digit
        auto_steps()         --Subfunction auto_steps() will automatically generate new values for "distance"
    until ((i == digits) or (steps == 0))
    message = ("Max. focus < inf:")
    x = start_distance
    print_focus()
    message = ("Found for infinite:")
    x = distance
    print_focus()
    console_redraw()
    sleep(3000)
    console_redraw()
    show_title = true
    show_meter = true
end

--Subfunction auto_steps() to search max. value for the current digit:
function auto_steps()
    local n = 1
    repeat
        distance = (start_distance + (steps * n))
        cls()
        j = i + 1
        print(string.format("steps:%07dmm, n:%d, i:%d", steps, n, j))
        message = "Testing:"
        x = distance
        print_focus()
        set_fixfoc()
        message = "Result: "
        x = focus
        print_focus()
        sleep(step_time)
        n = n + 1
    until (focus == inf) or (n == 11)
    if not ((n == 2) and (i == 0)) then  --Start counting digits (i) at first found value > 0
        i = i + 1
    end
    current_inf = distance
    start_distance = (distance - steps)
    current_maxfoc = start_distance
end

--Subfunction restore(), used at script aborting via keys SHOOT or MENU:
function restore()
    cls()
    if mf_mode == 1 then
        set_mf(0)         --Disable Auto-MF-mode
        print("Auto-MF disabled now")
    elseif afl_mode == 1 then
        set_aflock(0)     --Disable Auto-AFL-mode
        print("Auto-AFL disabled now")
    else                  --True with native activated MF-mode
        print("Native MF-mode kept")
    end
    console_redraw()
end

--Start main script
if tonumber(string.sub(string.gsub(get_buildinfo().build_number, "%p", ""), 1, 3)) >= 130 then  --Get CHDK version
    if not (get_focus_state() < 0) then    --True if cam is not in manual MF-mode
        mf = set_mf(1)     --Try to enable MF-mode 
        if mf == 1 then    --At cams with available MF-mode mf will be 1
            mf_mode = 1     
            print("Auto-MF enabled now")
        else
            set_aflock(1)  --Enable AFL-mode instead MF-mode
            afl_mode = 1
            print("Auto-AFL enabled now")
        end
    else
        print("Native MF-mode found")
    end
    capmode = require("capmode")  --Include lua-library capmode.lua from folder A/CHDK/LUALIB
    cmode = capmode.get_name()    --Get name of the used program
    if not (cmode == "AV") then
        ready = false
        errormessage1 = "Error: This script"
        errormessage2 = "requires program Av!"
    else
        --Start test to examine result for infinite focus if wanted:
        if inf_test == 1 then
            test_inf()
            ready = false
        end
    end
else       --CHDK version is to old; Error message and script aborting in main loop
    ready = false
    errormessage1 = "Error: This script"
    errormessage2 = "requires CHDK 1.3 or higher!"
end

--Start main loop:
if ready == true then
    print_steps()
    distance = start_distance
    set_fixfoc()                          --Set focus to choosen start value
    print("Script is ready now...")
    repeat
        wait_click(0)
        if is_pressed("up") then
            increase_steps()
        elseif is_pressed("down") then
            decrease_steps()
        elseif is_pressed("right") then
            distance = distance + steps
            if distance >= 9000000 then
                distance = 9000000
                print("Error: The max. distance")
                print("is limited to 9000 m!")
            end
            cls()
            set_fixfoc()
        elseif is_pressed("left") then
            distance = distance - steps
            if distance < 0 then
                distance = 0
                print("Error: The min. distance")
                print("is limited to 0 mm!")
            end
            cls()
            set_fixfoc()
        elseif is_pressed("set") then
            auto_test = 1
            cls()
            print("Start Auto-test for max. focus")
            print("distance below infinite")
            print("Press DISPLAY to start test:")
            print("Press MENU to abort test:")
            repeat
                wait_click(2000)       
                if is_pressed("display") then
                    auto_inf_search()
                    auto_test = 0   
                elseif is_pressed("menu") then
                    auto_test = 0
                    cls()
                    print("Auto-test for max. focus")                  
                    print("has been aborted!")
                    sleep(3000)
                end
                console_redraw()  
            until auto_test == 0
            auto_test = 1
        elseif is_pressed("display") then
            show_state()
        end
    until is_pressed("menu")
    restore()
else
    print(errormessage1)
    print(errormessage2)
    sleep(3000)
    console_redraw()
end
